@@ -189,10 +189,6 @@
   {{ return(load_result('list_schemas').table) }}
 {% endmacro %}
 
-{% macro hive__current_timestamp() -%}
-  current_timestamp()
-{%- endmacro %}
-
 {% macro hive__rename_relation(from_relation, to_relation) -%}
   {% call statement('rename_relation') -%}
     {% if not from_relation.type %}
@@ -273,4 +269,4 @@
   {% else %}
     {% do return('2') %}  {# assume hive 2 by default #}
   {% endif %}
-{% endmacro %}
\ No newline at end of file
+{% endmacro %}
